/*
 * Java port of Bullet (c) 2008 Martin Dvorak <jezek2@advel.cz>
 *
 * Bullet Continuous Collision Detection and Physics Library
 * Copyright (c) 2003-2007 Erwin Coumans  http://continuousphysics.com/Bullet/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package javabullet.collision.shapes;

import javabullet.linearmath.Transform;
import javax.vecmath.Vector3f;

/**
 * ConvexShape is an abstract shape interface.
 * It describes general convex shapes using the localGetSupportingVertex interface
 * used in combination with GJK or btConvexCast
 * 
 * @author jezek2
 */
public abstract class ConvexShape extends CollisionShape {

	public abstract Vector3f localGetSupportingVertex(Vector3f vec);

	//#ifndef __SPU__
	public abstract Vector3f localGetSupportingVertexWithoutMargin(Vector3f vec);

	//notice that the vectors should be unit length
	public abstract void batchedUnitVectorGetSupportingVertexWithoutMargin(Vector3f[] vectors, Vector3f[] supportVerticesOut, int numVectors);
	//#endif
	
	public abstract void getAabbSlow(Transform t, Vector3f aabbMin, Vector3f aabbMax);

	public abstract void setLocalScaling(Vector3f scaling);

	public abstract Vector3f getLocalScaling();

	public abstract void setMargin(float margin);

	public abstract float getMargin();

	public abstract int getNumPreferredPenetrationDirections();

	public abstract void getPreferredPenetrationDirection(int index, Vector3f penetrationVector);
	
}
